#include "TROOT.h"
#include "TTree.h"
#include "TChain.h"
#include "TFile.h"
#include "TH2F.h"
#include <string>  
#include <fstream> 
#include <sstream>
#include <iostream>
#include <vector>

using namespace std;

////////////////////////////////////////////////////////////////////////////////////
//
// C-Program to display events taken by the 37 detector stations of the GRANDE-Array
//
//  Displayed are:
//   - the charged energy deposits/station (lego- and color plots)
//   - the arrival times/station
//   - event information box
//
//  Input:
//   - ROOT file holding events measured by GRANDE downloaded from KCDC
//
//  called from root with:
//    root -l
//    .L EventDisplay_GRANDE-raw.C
//    run()
//
//  file: 'EventDisplay_GRANDE-raw.C'
//                                                             J. Wochele 28.11.2022
//
////////////////////////////////////////////////////////////////////////////////////

int run(void){
    gROOT->Reset();
    gStyle->SetOptStat(0);
    gStyle->SetPalette(55);
    gROOT->ProcessLine( "gErrorIgnoreLevel = 1001;");   // suppress ROOT warnings/Info

///////////////////////////////////////////////////////////////////////////////////
// inputs required:
// provide root file from KCDC DataShop or Preselections
// to display random events set 'single_switch' and 'cut_switch' to 'false'
// to display one event set 'single_switch' to 'true' and give run- and event-numbers
// to display events selected by cuts set 'cut_switch' to 'true' and define your cuts
// within the code below (search for 'cut_switch')
// to read only a given number of events set 'ievents' to the desired number
    
    std::string path_to_data = "./example_GRANDE-raw.root";

    bool single_switch = false;  // display only one single Event (give run and event numbers)
    int irun=5008;               // input run number to be displayed
    int iev=51111;               // input event number to be displayed

    bool cut_switch    = false; // display events matching cuts
    int ievents=0;               // number of events to be read from file
    
//////////////////////////////////////////////////////////////////////////////////
    
    std::cout<<"Input file:"<<path_to_data<<"\n";
     
    TChain array("array");
    TChain grande("grande");
    TChain general("general");
    TChain row("row_map");
    array.Add(path_to_data.c_str());
    grande.Add(path_to_data.c_str());
    general.Add(path_to_data.c_str());
    row.Add(path_to_data.c_str());

    int nentries = row.GetEntries();

  // general declarations
    int        general_id;
    int        run = 0;          // run number
    int        ev  = 0;          // event number
    int        gt;               // global event time
    double     p;                // Air Pressure
    double     t;                // Air Temperature
 
  // declarations of leaf types: array (GRANDE)
    int    array_id;
    double xc    = 0.;           // x-core reconstructed by KASCADE
    double yc    = 0.;           // y-cor reconstructed by KASCADE
    double az    = 0.;           // Az reconstructed by KASCADE
    double ze    = 0.;           // Ze reconstructed by KASCADE
    double ne    = 0.;           // number e/g [Size in ntuple] [log10]
    double age   = 0.;           // age reconstructed by KASCADE
    double nmu   = 0.;           // nr of muons reconstructed by KASCADE [log10]
    double en    = 0.;           // estimated primary energy [log10]
 
    int    edepositn;            // number of active e/g stations 
    int    edeposits[252];       // Station ID active e/g-Stations [edepositn]
    double edeposit[252];        // energy deposit of e/g stations [edepositn]
    int    mdepositn;            // number of active mu stations 
    int    mdeposits[192];       // Station ID active mu-Stations [mdepositn]
    double mdeposit[192];        // energy deposit of mu-stations [mdepositn]
    int    arrivaln;             // number of active AT stations 
    int    arrivals[252];        // Station ID particip AT-Stations [arrivaln]
    int    arrival[252]={-1000}; // arrival times to be filled in [arrivaln] 
    int    arrival1[252];        // aux variable
    char   datetime[20];         // Event Date and Time

  // GRANDE declarations
    int    grande_id;
    double xcg    = 0.;          // x-core reconstructed by Grande
    double ycg    = 0.;          // y-core reconstructed by Grande
    double azg    = 0.;          // Az-core reconstructed by Grande
    double zeg    = 0.;          // Ze-core reconstructed by Grande
    double nch    = 0.;          // number of charged part reconstructed by Grande
    double ageg   = 0.;          // age reconstructed by Grande
    double nmug   = 0.;          // number of muon derived from KASCADE

    int    gdepositn;            // nr of active Grande stations
    int    gdeposits[37];        // Station ID active Grande Stations [gdepositn]
    double gdeposit[37];         // energy deposit/Grande station [MeV] [gdepositn]
    int    garrivaln;            // nr of active Grande AT stations
    int    garrivals[37];        // Station ID active mu-Stations [mdepositn]
    int    garrival[37]={-1000}; // arrival time/Grande station [ns]
    int    garrival1[37];        // aux variable

    array.SetBranchStatus("*",  0);
    grande.SetBranchStatus("*", 0);
    general.SetBranchStatus("*",0);
 
    row.SetBranchStatus("array",  1);
    row.SetBranchStatus("grande", 1);
    row.SetBranchStatus("general",1);

    row.SetBranchAddress("array",   &array_id);
    row.SetBranchAddress("grande",  &grande_id);    
    row.SetBranchAddress("general", &general_id);

    general.SetBranchStatus("R",  1);
    general.SetBranchStatus("Ev", 1);
    general.SetBranchStatus("P",  1);
    general.SetBranchStatus("T",  1);
    general.SetBranchStatus("Gt", 1);

    array.SetBranchStatus("Xc",   1);
    array.SetBranchStatus("Yc",   1);
    array.SetBranchStatus("Az",   1);
    array.SetBranchStatus("Ze",   1);
    array.SetBranchStatus("Ne",   1);
    array.SetBranchStatus("Age",  1);
    array.SetBranchStatus("Nmu",  1);
    array.SetBranchStatus("E",    1);

    grande.SetBranchStatus("Xc",  1);
    grande.SetBranchStatus("Yc",  1);
    grande.SetBranchStatus("Az",  1);
    grande.SetBranchStatus("Ze",  1);
    grande.SetBranchStatus("Nch", 1);
    grande.SetBranchStatus("Age", 1);
    grande.SetBranchStatus("Nmu", 1);

    general.SetBranchStatus("EDepositN",  1);
    general.SetBranchStatus("EDeposit",   1);
    general.SetBranchStatus("EDepositS",  1);
    general.SetBranchStatus("MDepositN",  1);
    general.SetBranchStatus("MDeposit",   1);
    general.SetBranchStatus("MDepositS",  1);
    general.SetBranchStatus("ArrivalN",   1);
    general.SetBranchStatus("Arrival",    1);
    general.SetBranchStatus("ArrivalS",   1);
    general.SetBranchStatus("GDepositN",  1);
    general.SetBranchStatus("GDeposit",   1);
    general.SetBranchStatus("GDepositS",  1);
    general.SetBranchStatus("GArrivalN",  1);
    general.SetBranchStatus("GArrival",   1);
    general.SetBranchStatus("GArrivalS",  1);
    general.SetBranchStatus("Datetime",   1);
    
    general.SetBranchAddress("R",  &run);
    general.SetBranchAddress("Ev", &ev);
    general.SetBranchAddress("Gt", &gt); 
    general.SetBranchAddress("P",  &p); 
    general.SetBranchAddress("T",  &t); 

    array.SetBranchAddress("Xc",   &xc);
    array.SetBranchAddress("Yc",   &yc);
    array.SetBranchAddress("Az",   &az);
    array.SetBranchAddress("Ze",   &ze);
    array.SetBranchAddress("Ne",   &ne);
    array.SetBranchAddress("Age",  &age);
    array.SetBranchAddress("Nmu",  &nmu);
    array.SetBranchAddress("E",    &en);

    grande.SetBranchAddress("Xc",  &xcg);
    grande.SetBranchAddress("Yc",  &ycg);
    grande.SetBranchAddress("Az",  &azg);
    grande.SetBranchAddress("Ze",  &zeg);
    grande.SetBranchAddress("Nch", &nch);
    grande.SetBranchAddress("Age", &ageg);
    grande.SetBranchAddress("Nmu", &nmug);

    general.SetBranchAddress("EDepositN",  &edepositn);
    general.SetBranchAddress("EDeposit",   &edeposit);
    general.SetBranchAddress("EDepositS",  &edeposits);
    general.SetBranchAddress("MDepositN",  &mdepositn);
    general.SetBranchAddress("MDeposit",   &mdeposit);
    general.SetBranchAddress("MDepositS",  &mdeposits);
    general.SetBranchAddress("ArrivalN",   &arrivaln);
    general.SetBranchAddress("Arrival",    &arrival);
    general.SetBranchAddress("ArrivalS",   &arrivals);
    general.SetBranchAddress("GDepositN",  &gdepositn);
    general.SetBranchAddress("GDeposit",   &gdeposit);
    general.SetBranchAddress("GDepositS",  &gdeposits);
    general.SetBranchAddress("GArrivalN",  &garrivaln);
    general.SetBranchAddress("GArrival",   &garrival);
    general.SetBranchAddress("GArrivalS",  &garrivals);
    general.SetBranchAddress("Datetime",   &datetime);

    int Nc;
    int Nm;
    string Date, Time;
    char *s = new char[1];
    bool single_found = false;
    
  // Create histograms
    TH2F *hStNr      = new TH2F("hStNr","", 19,0,19, 21,0,21);  // GR Station ID histogram
    TH2F *hEnDepoCH  = new TH2F("hEnDepoCH",  "", 19,0,19, 21,0,21);  // ch-energy deposit 
    TH2F *hEnDepoCHc = new TH2F("hEnDepoCHc", "", 19,0,19, 21,0,21);  // ch-energy deposit 
    TH2F *hArrivalT  = new TH2F("hArrivalT",  "", 19,0,19, 21,0,21);  // Arrival Times 

  // define station ID GRANDE stations
   int StID[399] =
    {
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0,  0,  0,  0, 36,  0, 35,  0,  0, 34,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0, 37,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0, 33,  0,  0, 32,  0, 31,  0, 30,  0,  0,  0, 29,  0,  0, 28,  0,  0,
      
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0, 27,  0,  0, 26,  0, 25,  0,  0, 24,  0,  0, 23,  0,  0,  0,  0,
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0, 22,  0,  0, 21,  0,  0, 20,  0, 19,  0, 18,  0,  0,  0, 17,  0,  0,  0,
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      
      0,  0,  0,  0, 16,  0, 15,  0, 14,  0, 13,  0,  0,  0, 12,  0,  0,  0,  0,
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0,  0,  0, 10,  0,  9,  0,  8,  0,  0,  7,  0,  0,  0,  6,  0,  0,
      
      0,  0, 11,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0,  0,  5,  0,  4,  0,  3,  0,  2,  0,  0,  0,  1,  0,  0,  0,  0,
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    };
   
  // define station ID KASCADE stations
    int StIDA[256] =
    {      
    1,   2,    5,   6,  17,  18,  21,  22,  33,  34,  37,  38,  49,  50,  53,  54, 
    4,   3,    8,   7,  20,  19,  24,  23,  36,  35,  40,  39,  52,  51,  56,  55, 
    13,  14,   9,  10,  29,  30,  25,  26,  45,  46,  41,  42,  61,  62,  57,  58,
    16,  15,  12,  11,  32,  31,  28,  27,  48,  47,  44,  43,  64,  63,  60,  59,
 
    65,  66,  69,  70,  81,  82,  85,  86,   96,  97, 100, 101, 111, 112, 115, 116,
    68,  67,  72,  71,  84,  83,  88,  87,   99,  98, 103, 102, 114, 113, 118, 117, 
    77,  78,  73,  74,  92,  93,  89,  90,  108, 109, 104, 105, 123, 124, 119, 120,
    80,  79,  76,  75,  95,  94,  91,   0,    0, 110, 107, 106, 126, 125, 122, 121, 
 
    127, 128, 131, 132, 143, 144, 147,   0,   0, 158, 161, 162, 173, 174, 177, 178,
    130, 129, 134, 133, 146, 145, 149, 148, 160, 159, 164, 163, 176, 175, 180, 179,
    139, 140, 135, 136, 154, 155, 150, 151, 169, 170, 165, 166, 185, 186, 181, 182,
    142, 141, 138, 137, 157, 156, 153, 152, 172, 171, 168, 167, 188, 187, 184, 183,

    189, 190, 193, 194, 205, 206, 209, 210, 221, 222, 225, 226, 237, 238, 241, 242, 
    192, 191, 196, 195, 208, 207, 212, 211, 224, 223, 228, 227, 240, 239, 244, 243,
    201, 202, 197, 198, 217, 218, 213, 214, 233, 234, 229, 230, 249, 250, 245, 246, 
    204, 203, 200, 199, 220, 219, 216, 215, 236, 235, 232, 231, 252, 251, 248, 247 
    };

  // define display axes  
    hEnDepoCH->GetXaxis()->SetNdivisions(800);
    hEnDepoCH->GetXaxis()->SetTitle("Eastern direction #rightarrow    ");
    hEnDepoCH->GetXaxis()->SetTitleOffset(1.2);
    hEnDepoCH->GetYaxis()->SetNdivisions(800);
    hEnDepoCH->GetYaxis()->SetTitle(" #leftarrow  Northern direction   ");
    hEnDepoCH->GetYaxis()->SetTitleOffset(1.2);
    
    hEnDepoCHc->GetXaxis()->SetNdivisions(800);
    hEnDepoCHc->GetXaxis()->SetTitle("Eastern direction  #rightarrow    ");
    hEnDepoCHc->GetXaxis()->SetTitleOffset(1.2);
    hEnDepoCHc->GetYaxis()->SetNdivisions(800);
    hEnDepoCHc->GetYaxis()->SetTitle("Northern direction  #rightarrow  ");
    hEnDepoCHc->GetYaxis()->SetTitleOffset(1.2);
    
    hArrivalT->GetXaxis()->SetNdivisions(800);
    hArrivalT->GetXaxis()->SetTitle("Eastern direction #rightarrow   ");
    hArrivalT->GetXaxis()->SetTitleOffset(1.2);
    hArrivalT->GetYaxis()->SetNdivisions(800);
    hArrivalT->GetYaxis()->SetTitle(" #leftarrow  Northern direction");
    hArrivalT->GetYaxis()->SetTitleOffset(1.2);
    
  // Fill array with station IDs (and draw)
    for(int j=0; j<21; j++){
      for(int k=0; k<19; k++){
        int n = 19*j + k;
        hStNr->SetBinContent(k+1, j+1, 0);
        hStNr->SetBinContent(k+1, j+1, StID[n]);
      }
    }

    ostringstream title;
    title.str("");
    ostringstream ofile;
    ofile.str("");
    ostringstream infofile;
    infofile.str("");
    ostringstream label;
    label.str("");
    ofstream inffile;
    
 // print location of the KASCADE array
    TLine *array1 = new TLine(11.5,15,11.5,21);
    TLine *array2 = new TLine(11.5,15,17.5,15);
    TLine *array3 = new TLine(11.5,21,17.5,21);
    TLine *array4 = new TLine(17.5,15,17.5,21);
    array1->SetLineColor(4);
    array2->SetLineColor(4);
    array3->SetLineColor(4);
    array4->SetLineColor(4);
    array1->SetLineStyle(9);
    array2->SetLineStyle(9);
    array3->SetLineStyle(9);
    array4->SetLineStyle(9);
    array1->SetLineWidth(3);
    array2->SetLineWidth(3);
    array3->SetLineWidth(3);
    array4->SetLineWidth(3);
       
  // get entries and define Canvas
    std::cout << "Total number of events in file:  " <<nentries<<endl;
    
    auto c2 = new TCanvas("c2","Single Event Display",100,100,800,800);
    c2->Divide(2,2);

    if (ievents!=0) {
      nentries=ievents;
    }
    std::cout << "GRANDE events to be displayed: " << nentries <<endl;
    
  //******************************************************************  
  // entries loop

    for(int i=0; i<nentries; ++i){
      c2->Clear();
      c2->Divide(2,2);
      row.GetEntry(i);
      
// get branch_id's
      if (general_id<0) {
	continue;
      }
      if (general_id>=0) {
	general.GetEntry(general_id);
      }
      if (array_id>=0) {
	array.GetEntry(array_id);
      }
      if (grande_id>=0) {
	grande.GetEntry(grande_id);
      }


  // ========================================================================
  // event display GRANDE data
      if (grande_id>=0) {

	Date.assign(datetime,datetime+10);
	Time.assign(datetime+11,datetime+19);
	
  // mode to display a single event
      if (single_switch ) {
	if ( run != irun || ev != iev ) {
	  continue;
	} else {
	  single_found = true;
	}
      }

  // mode to display events matchen a cut
      if ( cut_switch ) {
	if ( nch <= 5.7 ) {
	  continue;
	}
      }
      std::cout << "displaying GRANDE event Run No: "<<run<<"  Event: "<<ev<<endl;

  // get number of KASCADE-Array muon stations used for GRANDE reconstruction
	Nm = 0;
	for(int j=0; j<16; j++){
	  for(int k=0; k<16; k++){
	    int n = 16*j + k;
	    int StInd = StIDA[n];                  // 1-252
	    for(int jmu=0; jmu<mdepositn; jmu++){
	      if ((mdeposit[jmu] >= 1.0) && (mdeposit[jmu] <= 600.0)) {
		if (mdeposits[jmu]==StInd) {
		  Nm = Nm + 1;
		}
	      }
	    }
	  }
	}

  //----------------------------------------------------------
  // display charged energy deposits (Box 1: Lego)

      title.str("");
      title << "'charged' energy deposits in [eV]";
      hEnDepoCH->SetTitle(title.str().c_str());
      
      c2->cd(1);
      c2->SetLeftMargin(0.15);     // distance to plot left
      c2->SetBottomMargin(0.15);   // distance to plot bottom
      c2->SetRightMargin(0.05);    // distance to plot right
      c2->SetTopMargin(0.15);      // distance to plot top
      c2->SetGridx();
      c2->SetGridy();

      Nc = 0;             

      for(int j=0; j<21; j++){
	for(int k=0; k<19; k++){
	  int n = 19*j + k;
	  int StInd = StID[n];                       // 1-37
	  hEnDepoCH->SetBinContent(k+1, j+1, 0.);      // clear 1st array
	  hEnDepoCHc->SetBinContent(k+1, j+1, 0.);     // clear 2nd array

	  for(int jeg=0; jeg<gdepositn; jeg++){
	    if (gdeposit[jeg] > 0.0) {
	      if (gdeposits[jeg]==StInd) {
		hEnDepoCH->SetBinContent(k+1, j+1, gdeposit[jeg]);
		hEnDepoCHc->SetBinContent(k+1, j+1, gdeposit[jeg]);
		Nc = Nc + 1;
	      }
	    }
	  }
	}
      }
      gPad->SetLogz(1);
      gPad->SetRightMargin(0.13);
      hEnDepoCH->GetYaxis()->SetTitle("#leftarrow   Northern direction   ");
      hEnDepoCH->Draw("lego2z");
      gPad->Update();

   // draw col-plot (Box2: color)
      c2->cd(2);
      title.str("");
      title << "'charged' energy deposits in [eV]";
      hEnDepoCHc->SetTitle(title.str().c_str());
      gPad->SetLogz(1);
      gPad->SetRightMargin(0.13);
      hEnDepoCHc->GetYaxis()->SetTitle("Northern direction   #rightarrow ");
      hEnDepoCHc->Draw("colz");
      gPad->Update();

   // draw location of KASCADE Array
      hStNr->Draw("textsame");
      array1->Draw("same");
      array2->Draw("same");
      array3->Draw("same");
      array4->Draw("same");
            
  //========================================================================
  // display arrival times (Box 3)
      title.str("");
      title << "Arrival time distribution in [ns]";
      hArrivalT->SetTitle(title.str().c_str());
      c2->cd(3);

  // calculate arrival time offset (for better representation)
      double Toffset = 10000.;
      for(int jat=0; jat<garrivaln; jat++){
	if (garrival[jat] <= Toffset) {
	  Toffset = garrival[jat];
	}
      }
      
      for(int j=0; j<21; j++){
	for(int k=0; k<19; k++){
	  int n = 19*j + k;
	  int StInd = StID[n];
	  hArrivalT->SetBinContent(k+1, j+1, 0.);   // clear data array
	  
	  for(int jat=0; jat<garrivaln; jat++){
	    garrival1[StInd] = 0.;
	    
	    if (garrivals[jat]==StInd) {
	      garrival1[StInd] = garrival[jat];
	      hArrivalT->SetBinContent(k+1, j+1, garrival1[StInd]-Toffset+50.);
	    }
	  }
	}
      }
      gPad->SetRightMargin(0.13);
      hArrivalT->Draw("lego2z");

      c2->Update();

      
  // ========================================================================
  // Show event information (Box 4)
      c2->cd(4);
      TLatex l;
      l.SetTextSize(0.07);
      l.SetTextFont(72);
      l.SetTextColor(kBlack);
      l.SetNDC(false);
      const double xLeft  = 0.15;
      const double xRight = 0.85;
      const double yTop = 0.85;
      const double dY = 0.06;
      double y = 0.90;      
      double x = 0.12;

  // draw info headline
      l.DrawLatex(x, y, ("GRANDE Event Information"));
      l.SetTextSize(0.045);
      l.SetTextFont(42);

  // draw quantity names - adjust left
      y = yTop;
      l.SetTextAlign(12);
      vector<string> columnLabels = { "Run Nr.", "Event Nr.", "Zenith Angle", "Azimuth Angle", "X-Core Position", "Y-Core Position", "lg(N_{ch})", "lg(N_{#mu})", "Number charged-Stations", "Number ARRAY #mu-Stations", "Number timing Stations"};
      for (const auto& col : columnLabels) {
	l.DrawLatex(xLeft, y, col.c_str());
	y -= dY;
      }
      double en = -1.00;            // only used for info file

  // draw quantities - adjust right
      y = yTop;
      l.SetTextAlign(32);
      vector<double> values;
      double xRun = (double)run;
      values.push_back(xRun);
      double xEv = (double)ev;
      values.push_back(xEv);
      values.push_back(zeg);
      values.push_back(azg);
      values.push_back(xcg);
      values.push_back(ycg);
      values.push_back(nch);
      values.push_back(nmug);

      double xNc = (double)Nc;
      values.push_back(xNc);
      double xNm = (double)Nm;
      values.push_back(xNm);
      double xTime = (double)garrivaln;
      values.push_back(xTime);

      vector<string> units = {"", "", "^{o}", "^{o}", " m", " m", "", "", "", "", ""};
      
      for (unsigned int iRow = 0; iRow < values.size(); ++iRow) {
	ostringstream cell;
	cell << fixed << setprecision((iRow <2 || iRow >7) ? 0 : 2) << values[iRow] << units[iRow];
	l.DrawLatex(xRight, y, cell.str().c_str());
	y -= dY;
      }
       
  // draw date and time strings
      double yTop1 = y;
      l.SetTextAlign(12);
      vector<string> columnLabels1 = { "Event Date", "Event Time"};
      for (const auto& col : columnLabels1) {
	l.DrawLatex(xLeft, y, col.c_str());
	y -= dY;
      }
      y = yTop1;
      l.SetTextAlign(32);
      vector<string> datim;
      datim.push_back(Date);
      datim.push_back(Time);
      for (unsigned int iRow = 0; iRow < datim.size(); ++iRow) {
	ostringstream cell;
	cell  << datim[iRow];
	l.DrawLatex(xRight, y, cell.str().c_str());
	y -= dY;
      }

      	c2->Update();

  // ========================================================================
  // print picture as .svg and event info file .info

      ofile.str("");
      ofile << "ED_GR_R" << std::setfill('0') << std::setw(4)<< run << "E"  << std::setfill('0') << std::setw(7) << ev;
      c2->Print(("./" + ofile.str() + ".svg").c_str());
      
  // print event info as txt file
      infofile.str("");
      infofile << "ED_GR_R" << std::setfill('0') << std::setw(4)<< run << "E"  << std::setfill('0') << std::setw(7) << ev;
      inffile.open("./" + infofile.str() + ".info");

      inffile << "Event Display GRANDE - Event information\n";
      inffile << "-------------------------------------------\n";
      inffile << "Run Number               " << run << " \n";
      inffile << "Event Number             " << ev << " \n";
      inffile << "Prim Energy (lg10)       " << fixed << setprecision(2) << en<< " eV\n";
      inffile << "Zenith Angle             " << zeg << "\u00B0\n";
      inffile << "Azimuth Angle            " << azg << "\u00B0\n";
      inffile << "X-Core Position          " << xcg << " m\n";
      inffile << "Y-Core Position          " << ycg << " m\n";
      inffile << "lg(Nch)                  " << nch << "\n";
      inffile << "lg(Nmu)                  " << nmug << "\n";
      inffile << "Number charged-Stations  " << Nc << "\n";
      inffile << "Number ARRAY \u03BC-Stations  "<< Nm << "\n";
      inffile << "Number timing stations   " << garrivaln << "\n";
      inffile << "Shower Age               " << ageg << "\n";
      inffile << "Event Time               " << Date << " " << Time << "\n";
      inffile << "Global Event Time        " << gt  << " (secs since 1.1.1970)\n";
      inffile << "Air Pressure             " << p  << " hPa\n";
      inffile << "Air Temperature          " << t  << "\u00B0\n\n";
      inffile.close();

      c2->Update();

  // get next event or quit
      if ( single_found ) {          // single event found
	return 0;
      }
      
      char* gets(s);                       // hit <return> to continue
      if (string(s)=='q')            // press 'q' to quit
	return 0;  
     
      }
    }
    return 0;
}
